package polyglot.ext.ibex.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.visit.*;
import polyglot.util.*;
import java.util.*;

/**
 * Tag interface for elements of a rule RHS.
 */
public class RhsCaseNode_c extends RhsNode_c implements RhsCaseNode {
    List elements;

    public RhsCaseNode_c(Position pos, List elements, Block action) {
        super(pos, action);
        this.elements = TypedList.copyAndCheck(elements, RhsElement.class, true);
    }

    public List allElements() { return elements(); }

    public List elements() { return elements; }
    public RhsCaseNode elements(List elements) {
        RhsCaseNode_c n = (RhsCaseNode_c) copy();
        n.elements = TypedList.copyAndCheck(elements, RhsElement.class, true);
        return n;
    }

    protected RhsCaseNode_c reconstruct(List elements, Block action) {
        if (! CollectionUtil.equals(elements, this.elements) ||
            action != this.action) {
            RhsCaseNode_c n = (RhsCaseNode_c) copy();
            n.elements = TypedList.copyAndCheck(elements, RhsElement.class, true);
            n.action = action;
            return n;
        }
        return this;
    }

    public Node visitChildren(NodeVisitor v) {
        List elements = visitList(this.elements, v);
        Block action = (Block) visitChild(this.action, v);
        return reconstruct(elements, action);
    }

    protected Node makeRhs(IbexTypeSystem ts) {
        List tags = new ArrayList(elements.size());
        List syms = new ArrayList(elements.size());

        List l = new ArrayList(elements.size());
        for (int i = 0; i < elements.size(); i++) {
            RhsElement n = (RhsElement) elements.get(i);
            if (n instanceof Tagged) {
                Tagged t = (Tagged) n;
                tags.add(t.tag());
            }
            else {
                tags.add(null);
            }

            syms.add(n.symbolInstance());
        }

        Rhs rhs = ts.rhsCase(position(), syms, tags);

        return rhs(rhs);
    }
}
