package polyglot.ext.ibex.lr;

import java.util.*;
import polyglot.main.Report;

/**
 * A wrapper around the SRGraph to treat it as an NFA.
 */
class SRNFA implements NFA {
    SRGraph srGraph;

    SRNFA(SRGraph srGraph) {
        this.srGraph = srGraph;
    }

    public NFAState start() { return null; }

    public List alphabet() { return srGraph.terminals; }

    public int size() { return srGraph.size(); }

    public Iterator succs(Node node) { return srGraph.succs(node); }
    public Iterator preds(Node node) { return srGraph.preds(node); }

    public Iterator nodes() { return srGraph.nodes(); }

    public BitSet accepting(NFAState q0) {
        BitSet a = srGraph.accepting();
        BitSet b = new BitSet();
        for (int i = a.nextSetBit(0); i >= 0; i = a.nextSetBit(i+1)) {
            if (srGraph.statesMap[i] == null)
                continue;
            for (Iterator k = srGraph.statesMap[i].iterator(); k.hasNext(); ) {
                SRNode n = (SRNode) k.next();
                b.set(n.index());
            }
        }
        return b;
    }

    public boolean accepting(NFAState q0, NFAState q) {
        SRNode n = (SRNode) q;
        return srGraph.accepting(n.state);
    }

    public NFAState nodeAt(int k) { return srGraph.nodeAt(k); }  

    public String toString() {
        return "SRNFA";
    }

    public void dump() {
        srGraph.dump();
    }
}
