package polyglot.ext.polyj.parse;

import polyglot.ext.polyj.ast.*;

import polyglot.ast.*;
import polyglot.visit.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.parse.*;

import java.util.*;

/**
 * An <code>Inst</code> represents a <code>Parse</code> of the form "P[T,U,...]".
 * Either there must be at least two types/expressions in the list, which implies that
 * this must be an instantiation of a polymorphic type, or there may be a single element
 * in the list that is known to be a type.
 * The type itself is still ambiguous.
 */
public class Inst extends Parse {
    TypeNode prefix;
    List params;

    public Inst(BaseParser parser, Position pos, TypeNode prefix, List params) throws Exception {
        super(parser, pos);
        this.prefix = prefix;
        this.params = TypedList.copyAndCheck(params, Sub.class, true); 

        if (prefix instanceof Inst) parser.die("Illegal instantiation", pos);
        if (prefix instanceof InstOrAccess) parser.die("Illegal instantiation", pos);
        if (prefix instanceof Access) parser.die("Illegal instantiation", pos);
    }

    public TypeNode toType() throws Exception {
        return nf.InstTypeNode(pos, prefix, AbstractSub.toParamList(params));
    }

    public Prefix toPrefix() throws Exception { 
        return toType(); 
    }
    
    public Receiver toReceiver() throws Exception { 
        return toType(); 
    }
    
    public QualifierNode toQualifier() throws Exception {
        return toType();
    }
    
    public TypeNode toClassType() throws Exception { 
        return toType(); 
    }
    
    public String toString() {
        return prefix + "" + params;
    }
    
    public Parse prepend(Parse base) throws Exception {
        List l = new LinkedList();
        l.add(this.toAmbSub());
        return new Inst(parser, pos, base.toType(), l);
    }

    public Receiver toParam() throws Exception {
        return toType();
    }
}
